@@ -1,6 +1,6 @@
 use std::alloc::Layout;
 
-use crate::ptr::{non_null::NonNull, reef::Ref};
+use crate::ptr::{NonNull, Ref};
 
 pub mod cpu;
 
@@ -23,7 +23,7 @@ pub trait Device {
 /// Defines the default allocator for a device.
 /// For instance, The default allocator for [`cpu::Cpu`] is [`std::alloc::Global`]
 pub trait DefaultDeviceAllocator: Device {
-    type Alloc: DeviceAllocator<Device = Self> + Default;
+    type Alloc: DeviceAllocator<Self> + Default;
 }
 
 /// Represents a type safe device-based pointer.
@@ -75,42 +75,36 @@ pub trait DevicePtr<T: ?Sized>: Copy {
 
 /// An Allocator in a specific device.
 /// All [`std::alloc::Allocator`]s are [`DeviceAllocator<Device=cpu::Cpu>`]
-pub trait DeviceAllocator {
+pub trait DeviceAllocator<D: Device + ?Sized> {
     #![allow(clippy::missing_safety_doc)]
 
     /// Error returned when failing to allocate
     type AllocError: std::error::Error;
 
-    /// Device that the allocations will live in.
-    type Device: Device;
+    fn allocate(&self, layout: Layout) -> Result<NonNull<[u8], D>, Self::AllocError>;
 
-    fn allocate(&self, layout: Layout) -> Result<NonNull<[u8], Self::Device>, Self::AllocError>;
+    fn allocate_zeroed(&self, layout: Layout) -> Result<NonNull<[u8], D>, Self::AllocError>;
 
-    fn allocate_zeroed(
-        &self,
-        layout: Layout,
-    ) -> Result<NonNull<[u8], Self::Device>, Self::AllocError>;
-
-    unsafe fn deallocate(&self, ptr: NonNull<u8, Self::Device>, layout: Layout);
+    unsafe fn deallocate(&self, ptr: NonNull<u8, D>, layout: Layout);
 
     unsafe fn grow(
         &self,
-        ptr: NonNull<u8, Self::Device>,
+        ptr: NonNull<u8, D>,
         old_layout: Layout,
         new_layout: Layout,
-    ) -> Result<NonNull<[u8], Self::Device>, Self::AllocError>;
+    ) -> Result<NonNull<[u8], D>, Self::AllocError>;
 
     unsafe fn grow_zeroed(
         &self,
-        ptr: NonNull<u8, Self::Device>,
+        ptr: NonNull<u8, D>,
         old_layout: Layout,
         new_layout: Layout,
-    ) -> Result<NonNull<[u8], Self::Device>, Self::AllocError>;
+    ) -> Result<NonNull<[u8], D>, Self::AllocError>;
 
     unsafe fn shrink(
         &self,
-        ptr: NonNull<u8, Self::Device>,
+        ptr: NonNull<u8, D>,
         old_layout: Layout,
         new_layout: Layout,
-    ) -> Result<NonNull<[u8], Self::Device>, Self::AllocError>;
+    ) -> Result<NonNull<[u8], D>, Self::AllocError>;
 }
