@@ -5,9 +5,9 @@ use std::{
     ops::{Deref, DerefMut},
 };
 
-use crate::ptr::reef::Ref;
+use crate::ptr::{NonNull, Ref};
 
-use super::{DefaultDeviceAllocator, Device, DeviceAllocator, DevicePtr, NonNull};
+use super::{DefaultDeviceAllocator, Device, DeviceAllocator, DevicePtr};
 
 #[derive(Debug)]
 /// The default device
@@ -63,9 +63,8 @@ impl<T: ?Sized> From<NonNull<T, Cpu>> for std::ptr::NonNull<T> {
     }
 }
 
-impl<A: Allocator> DeviceAllocator for A {
+impl<A: Allocator> DeviceAllocator<Cpu> for A {
     type AllocError = AllocError;
-    type Device = Cpu;
 
     fn allocate(&self, layout: Layout) -> Result<NonNull<[u8], Cpu>, AllocError> {
         self.allocate(layout).map(NonNull::from)
