@@ -8,24 +8,27 @@ use std::{
 use crate::{
     boxed::Box,
     device::{cpu::Cpu, DefaultDeviceAllocator, Device, DeviceAllocator, DevicePtr},
-    ptr::{non_null::NonNull, reef::Ref},
+    ptr::{NonNull, Ref},
     zero::Zero,
 };
 
 /// Same as [`std::vec::Vec`] but using device allocators rather than host allocators.
 /// This allows you to have owned buffers on GPUs and CPUs using a single data structure.
-pub struct Vec<T, A: DeviceAllocator = Global> {
-    buf: Box<[MaybeUninit<T>], A>,
+pub struct Vec<T, A: DeviceAllocator<D> = Global, D: Device = Cpu> {
+    buf: Box<[MaybeUninit<T>], A, D>,
     len: usize,
 }
 
-impl<T, A: DeviceAllocator> Drop for Vec<T, A> {
+/// A [`Vec`] that uses the default allocator for the device
+pub type DefaultVec<T, D = Cpu> = Vec<T, <D as DefaultDeviceAllocator>::Alloc, D>;
+
+impl<T, A: DeviceAllocator<D>, D: Device> Drop for Vec<T, A, D> {
     fn drop(&mut self) {
         unsafe {
             // drop the data
             if std::mem::needs_drop::<T>() {
                 // we are on the CPU
-                if A::Device::IS_CPU {
+                if D::IS_CPU {
                     let slice = &mut *(self.buf.ptr.as_ptr().as_raw());
                     let slice = &mut slice[..self.len];
                     for i in slice {
@@ -39,7 +42,7 @@ impl<T, A: DeviceAllocator> Drop for Vec<T, A> {
     }
 }
 
-impl<T: Copy, A: DeviceAllocator + Clone> Clone for Vec<T, A> {
+impl<T: Copy, A: DeviceAllocator<D> + Clone, D: Device> Clone for Vec<T, A, D> {
     fn clone(&self) -> Self {
         let slice = self.deref();
         unsafe {
@@ -51,12 +54,8 @@ impl<T: Copy, A: DeviceAllocator + Clone> Clone for Vec<T, A> {
     }
 }
 
-pub fn vec_from_host<T: Copy, D: DefaultDeviceAllocator>(slice: &[T]) -> Vec<T, D::Alloc> {
-    Vec::copy_from_host_in(slice, D::Alloc::default())
-}
-
-impl<T, A: DeviceAllocator> Vec<T, A> {
-    pub fn from_box(b: Box<[T], A>) -> Self {
+impl<T, A: DeviceAllocator<D>, D: Device> Vec<T, A, D> {
+    pub fn from_box(b: Box<[T], A, D>) -> Self {
         let len = b.len();
         unsafe { Self::from_raw_parts(b.into_uninit(), len) }
     }
@@ -98,11 +97,11 @@ impl<T, A: DeviceAllocator> Vec<T, A> {
 
     /// # Safety
     /// `buf` must be a valid allocation in `device`, and `len` items must be initialised
-    pub unsafe fn from_raw_parts(buf: Box<[MaybeUninit<T>], A>, len: usize) -> Self {
+    pub unsafe fn from_raw_parts(buf: Box<[MaybeUninit<T>], A, D>, len: usize) -> Self {
         Self { buf, len }
     }
 
-    pub fn into_raw_parts(self) -> (Box<[MaybeUninit<T>], A>, usize) {
+    pub fn into_raw_parts(self) -> (Box<[MaybeUninit<T>], A, D>, usize) {
         let v = ManuallyDrop::new(self);
         unsafe { (std::ptr::read(&v.buf), v.len) }
     }
@@ -131,7 +130,7 @@ impl<T, A: DeviceAllocator> Vec<T, A> {
         self.buf.len()
     }
 
-    pub fn space_capacity_mut(&mut self) -> &mut Ref<[MaybeUninit<T>], A::Device> {
+    pub fn space_capacity_mut(&mut self) -> &mut Ref<[MaybeUninit<T>], D> {
         &mut self.buf.deref_mut()[self.len..]
     }
 
@@ -186,51 +185,51 @@ impl<T, A: Allocator> From<Vec<T, A>> for std::vec::Vec<T, A> {
     }
 }
 
-impl<T, A: Allocator> Vec<T, A> {
+impl<T, A: Allocator> Vec<T, A, Cpu> {
     pub fn into_std(self) -> std::vec::Vec<T, A> {
         self.into()
     }
 }
 
-impl<T, A: DeviceAllocator> Deref for Vec<T, A> {
-    type Target = Ref<[T], A::Device>;
+impl<T, A: DeviceAllocator<D>, D: Device> Deref for Vec<T, A, D> {
+    type Target = Ref<[T], D>;
 
     fn deref(&self) -> &Self::Target {
         unsafe { self.buf.deref()[..self.len()].assume_init() }
     }
 }
 
-impl<T, A: DeviceAllocator> DerefMut for Vec<T, A> {
+impl<T, A: DeviceAllocator<D>, D: Device> DerefMut for Vec<T, A, D> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         unsafe { self.buf.deref_mut()[..self.len].assume_init_mut() }
     }
 }
 
-impl<T, A: DeviceAllocator> Borrow<Ref<[T], A::Device>> for Vec<T, A> {
-    fn borrow(&self) -> &Ref<[T], A::Device> {
+impl<T, A: DeviceAllocator<D>, D: Device> Borrow<Ref<[T], D>> for Vec<T, A, D> {
+    fn borrow(&self) -> &Ref<[T], D> {
         self
     }
 }
 
-impl<T, A: DeviceAllocator> AsRef<Ref<[T], A::Device>> for Vec<T, A> {
-    fn as_ref(&self) -> &Ref<[T], A::Device> {
+impl<T, A: DeviceAllocator<D>, D: Device> AsRef<Ref<[T], D>> for Vec<T, A, D> {
+    fn as_ref(&self) -> &Ref<[T], D> {
         self
     }
 }
 
-impl<T, A: DeviceAllocator<Device = Cpu>> AsRef<[T]> for Vec<T, A> {
+impl<T, A: DeviceAllocator<Cpu>> AsRef<[T]> for Vec<T, A, Cpu> {
     fn as_ref(&self) -> &[T] {
         self
     }
 }
 
-impl<T, A: DeviceAllocator> AsMut<Ref<[T], A::Device>> for Vec<T, A> {
-    fn as_mut(&mut self) -> &mut Ref<[T], A::Device> {
+impl<T, A: DeviceAllocator<D>, D: Device> AsMut<Ref<[T], D>> for Vec<T, A, D> {
+    fn as_mut(&mut self) -> &mut Ref<[T], D> {
         self
     }
 }
 
-impl<T, A: DeviceAllocator<Device = Cpu>> AsMut<[T]> for Vec<T, A> {
+impl<T, A: DeviceAllocator<Cpu>> AsMut<[T]> for Vec<T, A, Cpu> {
     fn as_mut(&mut self) -> &mut [T] {
         self
     }
